from __future__ import annotations
import jinja2
from typing import Callable
import os
from LatexTemplater.LatexGenerator import PdfLatexGenerator, LatexGenerator
from typing import List
import json
import glob


class TemplateCore:
    """
    A Jinja template class made to work well with latex the following commands
    are used to render text:
    \\PYTHON{} for blocks
    \\PY{} for variables
    \\#{} for comments

    custom filters can be written for all sorts of things
    """
    __instance = None

    @staticmethod
    def instance() -> TemplateCore:
        """
        Gets the globa instance of the Template that all plugins register in
        """
        if TemplateCore.__instance is None:
            TemplateCore.__instance = TemplateCore()
        return TemplateCore.__instance

    def __init__(self) -> None:
        self._templater = jinja2.Environment(
            block_start_string=r'\PYTHON{',
            block_end_string='}',
            variable_start_string=r'\PY{',
            variable_end_string='}',
            comment_start_string='#{',
            comment_end_string='}',
            line_statement_prefix='%-',
            line_comment_prefix='%#',
            trim_blocks=True,
            autoescape=False,
            loader=jinja2.FileSystemLoader(".")
        )
        self._resultsFolder = '.'
        self._generator = PdfLatexGenerator()

    @property
    def resultsFolder(self) -> str:
        return self._resultsFolder

    @resultsFolder.setter
    def resultsFolder(self, newResultFolder: str):
        self._resultsFolder = newResultFolder

    @property
    def templateDir(self) -> str:
        return self._templater.loader.searchpath

    @templateDir.setter
    def templateDir(self, templateFolder: str) -> None:
        self._templater.loader = jinja2.FileSystemLoader(
            os.path.abspath(templateFolder)
        )

    @property
    def generator(self) -> LatexGenerator:
        return self._generator

    @generator.setter
    def generator(self, newGenerator: LatexGenerator) -> None:
        self._generator = newGenerator

    def registerFilter(self,
                       name: str,
                       filter: Callable[[any], str]) -> None:
        """
        This function is used to register a jinja filter for
        the templates
        """
        self._templater.filters[name] = filter

    def filter(self, filterName: str, arg: any) -> str:
        return self._templater.filters[filterName](arg)

    def render(self, templateFile: str, varFile: str = None, **args) -> None:
        """
        render a template file and output it to output file. template file
        should be a path relative to template folder mentioned above. Will
        place it in the resultsFolder. If no file extension is included the
        defeault extension will be chosen(normally .tex.j2). The last File
        extension will be dropped, for the ouput file
        Variables can also be specified in a file and automatically read in
        if both are present, args takes presidence
        (typically j2)
        example:
         - render("test")
        inputFile is interpreted as test.tex.j2
        outputFIle is test.tex
        """
        vars = {}
        if varFile is not None:
            with open(varFile) as jsonFile:
                data = json.load(jsonFile)
                vars.update(data)
        vars.update(args)

        if "." not in templateFile:
            templateFile += ".tex.j2"
        print(f'{templateFile}, {os.path.exists(templateFile)}')
        template = self._templater.get_template(templateFile)
        outFileName = '.'.join(templateFile.split('.')[:-1])

        if not os.path.isdir(self.resultsFolder):
            os.mkdir(self.resultsFolder)
        with open(f'{self.resultsFolder}/{outFileName}', "w") as outFile:
            outFile.write(template.render(**args))

    def generate(self,
                 mainTexFile: str = None,
                 resultDir: str = None,
                 render: bool = True, **args) -> None:
        templateFiles = glob.glob(self.templateDir[0] + "/*.tex.j2")
        templateFiles = [path.split('/')[-1] for path in templateFiles]
        print(f'texFile= {mainTexFile}')
        if render:
            for templateFile in templateFiles:
                self.render(templateFile, **args)

        if mainTexFile is None:
            mainTexFile = templateFiles[0]

        if "." not in mainTexFile:
            mainTexFile += ".tex"

        if resultDir is None:
            resultDir = self._resultsFolder
        self.generator.generate(mainTexFile, resultDir, resultDir)



from LatexTemplater.TemplateFilter import TemplateFilter
from LatexTemplater.TemplateCore import TemplateCore
from typing import Dict, Callable


def registrationInfo() -> Dict[str, Callable[[any], str]]:
    """
    Returns all of the infromation needed for registration of filters
    in this file
    """
    return {
        ImaginaryFilter.name: ImaginaryFilter.filter,
    }


class ImaginaryFilter(TemplateFilter):
    """
    Filters a Imaginary Number
    """

    name = "imaginary"

    @staticmethod
    def filter(val: complex) -> str:
        inst = TemplateCore.instance()
        return (f'{inst.filter("decimal", val.real)} + ' +
                f'{inst.filter("decimal", val.imag)}j')



import argparse
from LatexTemplater.TemplateCore import TemplateCore


def argParse():
    parser = argparse.ArgumentParser(prog="LatexTemplater",
                                     description="A latex Templating system "
                                     "that allows for easy injection of python "
                                     "into latex")
    parser.add_argument('main_template_file', type=str, nargs=1,
                        help="Path to the main tex file")
    parser.add_argument('--path', "-p", type=str, nargs=1, default=".",
                        help="Path to any other helper files")
    parser.add_argument('--vars', '-v', type=str, nargs=1, default=None,
                        help="Path to a config file which specifies all "
                        "variables")
    parser.add_argument('--output', '-o', type=str, nargs=1, default=".",
                        help="output directory for generated latex and pdf")
    return parser


def main():
    parser = argParse()
    inst = TemplateCore.instance()
    args = parser.parse_args()
    inst.templateDir = args.path
    inst.generate(args.main_template_file,
                  args.output,
                  render=True,
                  varFile=args.vars)



from abc import ABC, abstractmethod


class TemplateFilter(ABC):
    """
    The interface type for a filter
    """

    @abstractmethod
    def filter(any: any) -> str:
        """
        A filter that is used by jinja to mutate data
        """
        pass



from LatexTemplater.TemplateFilter import TemplateFilter
from LatexTemplater.TemplateCore import TemplateCore
import os
from typing import List, Dict, Callable


def registrationInfo() -> Dict[str, Callable[[any], str]]:
    """
    Returns all of the infromation needed for registration of filters
    in this file
    """
    return {
        BMatrixFilter.name: BMatrixFilter.filter,
        VMatrixFilter.name: VMatrixFilter.filter,
    }


def _matrix(values: List[List[float]], matrixType: str) -> str:
    """
    A utility function that is used to print a generic matrix
    """
    inst = TemplateCore.instance()
    return_str = r"\begin{" + matrixType + "}" + os.linesep
    for i, row in enumerate(values):
        for j, item in enumerate(row):
            if type(item) == float:
                return_str += inst.filter("decimal", item)
            elif type(item) == complex:
                return_str += inst.filter("imaginary", item)
            else:
                return_str += str(item)
            if not (j == len(row)-1):
                return_str += " & "
        if not (i == len(values)-1):
            return_str += r"\\"
        return_str += os.linesep
    return_str += r"\end{" + matrixType + "}" + os.linesep
    return return_str


class BMatrixFilter(TemplateFilter):
    """
    Filters a float value
    """

    name = "bmatrix"

    @staticmethod
    def filter(bmatrix: List[List[float]]) -> str:
        return _matrix(bmatrix, "bmatrix")


class VMatrixFilter(TemplateFilter):
    """
    Filters a float value
    """

    name = "vmatrix"

    @staticmethod
    def filter(bmatrix: List[List[float]]) -> str:
        return _matrix(bmatrix, "vmatrix")



import importlib
from typing import List


class PluginInterface:
    """
    Defines the interface that is used by LatexTemplater for custom filters
    """

    @staticmethod
    def initialize():
        """
        Initializes the plugins with Latex templater
        """


def load_plugins(plugins: List[str]) -> None:
    """
    Loads the plugins defined in the plugins list, will try the listed, if that 
    doesnt work then it will try same name with .Plugin appended
    """
    for plugin_name in plugins:
        plugin = import_module(plugin_name)
        try:
            plugin.initialize()
        except AttributeError:
            plugin = import_module(plugin_name + '.Plugin')
            plugin.initialize()


def import_module(name: str) -> PluginInterface:
    return importlib.import_module(name)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="LatexTemplater",
    version="0.4.7",
    author="Nathan Rose",
    author_email="mail.nathanrose@gmail.com",
    description="A package to allow python templating in latex files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/NathanRoseCE/LatexTemplater',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    install_requires=[
        'ninja2',
    ],
    entry_points={
        'console_scripts': [
            'sample=ArgParser:main',
        ],
    },
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)



from LatexTemplater.TemplateCore import TemplateCore
from . import Decimal
from LatexTemplater.CoreFilters import Imaginary
from LatexTemplater.CoreFilters import Equation
from LatexTemplater.CoreFilters import Matrix
from LatexTemplater.CoreFilters import InlineEquation


def initialize():
    """
    Registers all the types with the latex filter
    """
    instance = TemplateCore.instance()
    registrationFuncs = [
        Decimal.registrationInfo,
        Imaginary.registrationInfo,
        Equation.registrationInfo,
        Matrix.registrationInfo,
        InlineEquation.registrationInfo
    ]
    for registerFunc in registrationFuncs:
        filterinfo = registerFunc()
        for name, filter in filterinfo.items():
            instance.registerFilter(name, filter)



import abc
import subprocess
import shutil
import glob
import os


class LatexGenerator(abc.ABC):
    """
    Base class that is used to consume Tex files and produce pdf files
    """

    @abc.abstractmethod
    def generate(self,
                 main_tex_file: str,
                 tex_folder: str = None,
                 texFolder: str = None) -> None:
        """
        Generates the pdf file at the resultFile location given the main_tex
        file if main_tex_file does not have a file extension .tex is assumed,
        if result file is not specified, {main_tex_file}.pdf is assumed
        """
        pass


class PdfLatexGenerator:
    """
    A Latex Generator for PDFLatex, requires pdflatex to be installed in order
    to be used
    """

    def generate(self,
                 main_tex_file: str,
                 tex_folder: str = None,
                 resultFolder: str = None) -> None:
        """
        Override of LatexGenerator.generate
        """
        print(f'{tex_folder}, {main_tex_file}')
        print(f'{os.path.join(tex_folder, main_tex_file)}')
        process = subprocess.Popen(['pdflatex', f'{os.path.join(tex_folder, main_tex_file)}'],
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        stdout, stderro = process.communicate()
        exitCode = process.returncode
        for file in glob.glob(f"{main_tex_file.split('.')[0]}.*"):
            shutil.move(file, os.path.join(resultFolder, file))
        if exitCode != 0:
            raise Exception(stdout)
